"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.parseAndExtractConfig = parseAndExtractConfig;
const fs = require('fs');
const path = require('path');
const babel = require('@babel/parser');
const generate = require('@babel/generator').default;
const babelPresetTypeScript = require('@babel/preset-typescript');
const traverse = require('@babel/traverse').default;
const types = require('@babel/types');
const {
  convertStyledToStyledVerbosed,
  convertSxToSxVerbosed
} = require('@gluestack-style/react/lib/commonjs/convertSxToSxVerbosed');
const {
  propertyTokenMap
} = require('@gluestack-style/react/lib/commonjs/propertyTokenMap');
const {
  stableHash
} = require('@gluestack-style/react/lib/commonjs/stableHash');
const {
  CSSPropertiesMap,
  reservedKeys
} = require('@gluestack-style/react/lib/commonjs/core/styled-system');
const {
  StyleInjector
} = require('@gluestack-style/react/lib/commonjs/style-sheet/index');
const {
  updateOrderUnResolvedMap
} = require('@gluestack-style/react/lib/commonjs/updateOrderUnResolvedMap');
const {
  deepMerge
} = require('@gluestack-style/react/lib/commonjs/utils');
const {
  setObjectKeyValue
} = require('@gluestack-style/react/lib/commonjs/core/utils');
const {
  checkAndReturnUtilityProp
} = require('@gluestack-style/react/lib/commonjs/core/convert-utility-to-sx');
const {
  generateMergedThemeTokens,
  resolvePlatformTheme
} = require('@gluestack-style/react/lib/commonjs/utils');
const BUILD_TIME_GLUESTACK_STYLESHEET = new StyleInjector();
let configThemePath = [];
let ConfigDefault = {};
const convertExpressionContainerToStaticObject = (properties, result = {}, keyPath = [], propsToBePersist = {}) => {
  try {
    properties === null || properties === void 0 || properties.forEach((property, index) => {
      const nodeName = property.key.name ?? property.key.value;
      if (property.value.type === 'ObjectExpression') {
        keyPath.push(nodeName);
        convertExpressionContainerToStaticObject(property.value.properties, result, keyPath, propsToBePersist);
        keyPath.pop();
      } else if (property.value.type === 'Identifier') {
        if (property.key.value) {
          setObjectKeyValue(propsToBePersist, [...keyPath, nodeName], property.value.name);
        }
        if (property.key.name) {
          setObjectKeyValue(propsToBePersist, [...keyPath, nodeName], property.value.name);
        }
      } else {
        if (property.key.value) {
          setObjectKeyValue(result, [...keyPath, property.key.value], property.value.value);
        }
        if (property.key.name) {
          setObjectKeyValue(result, [...keyPath, property.key.name], property.value.value);
        }
      }
    });
    return {
      result,
      propsToBePersist
    };
  } catch (err) {
    throw new Error(`Error while converting expression container to static object. Error: ${err.message} Stack: ${err.stack}.`);
  }
};
function findThemeAndComponentConfig(node) {
  let themeNode = null;
  let componentConfigNode = null;
  node.forEach(prop => {
    const propKey = prop.key.name ? prop.key.name : prop.key.value;
    if (propKey === 'theme') {
      themeNode = prop;
    } else if (propKey === 'componentConfig') {
      componentConfigNode = prop;
    }
  });
  return {
    themeNode,
    componentConfigNode
  };
}
function addQuotesToObjectKeys(code) {
  const ast = babel.parse(`var a = ${code}`, {
    presets: [babelPresetTypeScript],
    plugins: ['typescript'],
    sourceType: 'module'
  });
  traverse(ast, {
    ObjectProperty: objectPropertyPath => {
      if (types.isTemplateLiteral(objectPropertyPath.node.value)) {
        objectPropertyPath.node.value = types.stringLiteral(objectPropertyPath.node.value.quasis[0].value.raw);
      }
      if (types.isIdentifier(objectPropertyPath.node.key)) {
        objectPropertyPath.node.key = types.stringLiteral(objectPropertyPath.node.key.name);
      }
      if (types.isNumericLiteral(objectPropertyPath.node.key)) {
        objectPropertyPath.node.key = types.stringLiteral(objectPropertyPath.node.key.extra.raw);
      }
      if (types.isStringLiteral(objectPropertyPath.node.value)) {
        objectPropertyPath.node.value = types.stringLiteral(objectPropertyPath.node.value.value);
      }
    }
  });
  let initAst;
  traverse(ast, {
    ObjectProperty: objectPropertyPath => {
      var _objectPropertyPath$n, _objectPropertyPath$n2, _objectPropertyPath$n3, _objectPropertyPath$n4;
      if (types.isArrayExpression(objectPropertyPath === null || objectPropertyPath === void 0 || (_objectPropertyPath$n = objectPropertyPath.node) === null || _objectPropertyPath$n === void 0 ? void 0 : _objectPropertyPath$n.value)) {
        let arrayElements = objectPropertyPath.node.value.elements;
        const dynamicElementsIndex = [];
        arrayElements.forEach((element, index) => {
          if (types.isNewExpression(element) || types.isIdentifier(element) || types.isTemplateLiteral(element)) {
            dynamicElementsIndex.push(index);
          }
        });
        arrayElements = arrayElements.filter((element, index) => !dynamicElementsIndex.includes(index));
        objectPropertyPath.node.value.elements = arrayElements;
      } else if (types.isIdentifier(objectPropertyPath === null || objectPropertyPath === void 0 || (_objectPropertyPath$n2 = objectPropertyPath.node) === null || _objectPropertyPath$n2 === void 0 ? void 0 : _objectPropertyPath$n2.value) || types.isTemplateLiteral(objectPropertyPath === null || objectPropertyPath === void 0 || (_objectPropertyPath$n3 = objectPropertyPath.node) === null || _objectPropertyPath$n3 === void 0 ? void 0 : _objectPropertyPath$n3.value) || types.isConditionalExpression(objectPropertyPath === null || objectPropertyPath === void 0 || (_objectPropertyPath$n4 = objectPropertyPath.node) === null || _objectPropertyPath$n4 === void 0 ? void 0 : _objectPropertyPath$n4.value)) {
        objectPropertyPath.remove();
      }
    }
  });
  traverse(ast, {
    VariableDeclarator: variableDeclaratorPath => {
      initAst = variableDeclaratorPath.node.init;
    }
  });
  const {
    code: output
  } = generate(initAst, {
    sourceType: 'module',
    presets: [babelPresetTypeScript],
    plugins: ['typescript']
  });
  return output;
}
const merge = require('lodash.merge');
function isReservedKey(key, variantProps = []) {
  return Object.values(reservedKeys).some(obj => obj.key === key) || key === 'variants' || key === 'compoundVariants' || key === 'props' || key === 'defaultProps' || key.startsWith('_') || variantProps.includes(key);
}
function getSkippedPluginProps(theme, variantProps = [], keyPath = [], skippedProps = {}, themeProps = {}) {
  Object.keys(theme).forEach(themeKey => {
    if (typeof theme[themeKey] === 'object') {
      if (isReservedKey(themeKey, variantProps)) {
        keyPath.push(themeKey);
        getSkippedPluginProps(theme[themeKey], variantProps, keyPath, skippedProps, themeProps);
        keyPath.pop();
      } else {
        keyPath.push(themeKey);
        setObjectKeyValue(skippedProps, keyPath, theme[themeKey]);
        keyPath.pop();
      }
    } else {
      keyPath.push(themeKey);
      setObjectKeyValue(themeProps, keyPath, theme[themeKey]);
      keyPath.pop();
    }
  });
  return {
    themeProps,
    skippedProps
  };
}

// Function to find a property by key path
function findProperty(obj, keyPath) {
  if (keyPath.length === 0) return obj;
  let key = keyPath[0];
  let prop = obj.properties.find(prop => prop.key.value === key);
  if (prop && keyPath.length > 1) {
    return findProperty(prop.value, keyPath.slice(1));
  } else {
    return prop;
  }
}

// Function to merge properties of two objects
function mergeProperties(obj1, obj2, keyPath = []) {
  obj2.properties.forEach(prop2 => {
    let newKeyPath = [...keyPath, prop2.key.value];
    let prop1 = findProperty(obj1, newKeyPath);
    if (prop1) {
      if (types.isObjectExpression(prop1.value) && types.isObjectExpression(prop2.value)) {
        // If both properties are objects, merge them recursively
        mergeProperties(prop1.value, prop2.value, newKeyPath);
      } else {
        // If the properties are not objects, replace the property in obj1 with the property from obj2
        prop1.value = prop2.value;
      }
    } else {
      // If the property does not exist in obj1, add it
      let parentObj = findProperty(obj1, keyPath);
      if (parentObj && parentObj.type === 'ObjectExpression') {
        parentObj.properties.push(types.objectProperty(types.stringLiteral(prop2.key.value), prop2.value));
      } else {
        // Handle the case where the parent object doesn't exist
        console.error(`Parent object not found for key path: ${keyPath.join('.')}`);
      }
    }
  });
}

// // Function to merge properties of two objects
// function mergeProperties(obj1, obj2) {
//   obj2.properties.forEach((prop2) => {
//     let prop1 = obj1.properties.find(
//       (prop1) => prop1.key.value === prop2.key.value
//     );

//     if (prop1) {
//       if (
//         types.isObjectExpression(prop1.value) &&
//         types.isObjectExpression(prop2.value)
//       ) {
//         // If both properties are objects, merge them recursively
//         mergeProperties(prop1.value, prop2.value);
//       } else {
//         // If the properties are not objects, replace the property in obj1 with the property from obj2
//         prop1.value = prop2.value;
//       }
//     } else {
//       // If the property does not exist in obj1, add it
//       obj1.properties.push(prop2);
//     }
//   });
// }

function mergeASTs(ast1, obj) {
  if (Object.keys(obj).length === 0) return ast1;
  let mergedAST = null;
  const ast2 = babel.parse(`var a = ${JSON.stringify(obj)}`, {
    presets: [babelPresetTypeScript],
    plugins: ['typescript'],
    sourceType: 'module'
  });
  traverse(ast2, {
    // For each ObjectExpression in the second AST
    ObjectExpression(objectExpressionPath) {
      // Merge the properties of the objects
      mergeProperties(ast1, objectExpressionPath.node);
      objectExpressionPath.stop();
    }
  });
  const {
    code: output
  } = generate(ast1);
  const outputAST = babel.parse(`var a = ${output}`, {
    presets: [babelPresetTypeScript],
    plugins: ['typescript'],
    sourceType: 'module'
  });
  traverse(outputAST, {
    VariableDeclarator(variableDeclaratorPath) {
      mergedAST = variableDeclaratorPath.node.init;
    }
  });
  return mergedAST;
}
function getBuildTimeParams(theme, componentConfig, extendedConfig, outputLibrary, platform, type, disableExtraction) {
  var _ConfigDefault;
  resolvePlatformTheme(theme, process.env.GLUESTACK_STYLE_TARGET);
  const mergedPropertyConfig = {
    ...((_ConfigDefault = ConfigDefault) === null || _ConfigDefault === void 0 ? void 0 : _ConfigDefault.propertyTokenMap),
    ...propertyTokenMap
  };
  const componentExtendedConfig = merge({}, {
    ...ConfigDefault,
    propertyTokenMap: {
      ...mergedPropertyConfig
    }
  });
  if (theme && Object.keys(theme).length > 0) {
    let verbosedTheme = {};
    const variantProps = [];
    Object.keys((theme === null || theme === void 0 ? void 0 : theme.variants) ?? {}).forEach(key => {
      var _theme$variants;
      variantProps.push(key);
      Object.keys(theme === null || theme === void 0 || (_theme$variants = theme.variants) === null || _theme$variants === void 0 ? void 0 : _theme$variants[key]).forEach(prop => {
        variantProps.push(prop);
      });
    });
    const {
      skippedProps,
      themeProps
    } = getSkippedPluginProps(theme, variantProps);
    verbosedTheme = convertStyledToStyledVerbosed(themeProps);
    let componentHash = stableHash({
      ...verbosedTheme,
      ...componentConfig,
      ...extendedConfig
    });
    if (outputLibrary) {
      componentHash = outputLibrary + '-' + componentHash;
    }
    const {
      styledIds,
      verbosedStyleIds
    } = updateOrderUnResolvedMap(verbosedTheme, componentHash, type, componentConfig, BUILD_TIME_GLUESTACK_STYLESHEET, platform);
    const toBeInjected = BUILD_TIME_GLUESTACK_STYLESHEET.resolve(styledIds, componentExtendedConfig, {});
    const current_global_map = BUILD_TIME_GLUESTACK_STYLESHEET.getStyleMap();
    const orderedResolvedTheme = [];
    current_global_map === null || current_global_map === void 0 || current_global_map.forEach(styledResolved => {
      var _styledResolved$meta;
      if (styledIds.includes(styledResolved === null || styledResolved === void 0 || (_styledResolved$meta = styledResolved.meta) === null || _styledResolved$meta === void 0 ? void 0 : _styledResolved$meta.cssId)) {
        orderedResolvedTheme.push(styledResolved);
      }
    });
    const styleIdsAst = generateObjectAst(verbosedStyleIds);
    const toBeInjectedAst = generateObjectAst(toBeInjected);
    const orderedResolvedAst = generateArrayAst(orderedResolvedTheme);
    const orderedStyleIdsArrayAst = types.arrayExpression(styledIds === null || styledIds === void 0 ? void 0 : styledIds.map(cssId => types.stringLiteral(cssId)));
    const resultParamsNode = types.objectExpression([types.objectProperty(types.stringLiteral('orderedResolved'), orderedResolvedAst), types.objectProperty(types.stringLiteral('disableExtraction'), types.booleanLiteral(disableExtraction)), types.objectProperty(types.stringLiteral('toBeInjected'), toBeInjectedAst), types.objectProperty(types.stringLiteral('styledIds'), orderedStyleIdsArrayAst), types.objectProperty(types.stringLiteral('verbosedStyleIds'), styleIdsAst)]);
    return {
      node: resultParamsNode,
      styles: toBeInjected,
      skippedThemeProps: skippedProps
    };
  }
  return {
    node: null,
    styles: {},
    skippedThemeProps: {}
  };
}
function replaceSingleQuotes(str) {
  let inDoubleQuotes = false;
  let newStr = '';
  for (let i = 0; i < str.length; i++) {
    if (str[i] === '"') {
      inDoubleQuotes = !inDoubleQuotes;
    }
    if (str[i] === "'" && !inDoubleQuotes) {
      newStr += '"';
    } else {
      newStr += str[i];
    }
  }
  return newStr;
}
function getObjectFromAstNode(node) {
  var _objectCode;
  let objectCode = generate(node).code;
  objectCode = (_objectCode = objectCode) === null || _objectCode === void 0 ? void 0 : _objectCode.replace(/as const/g, '');
  objectCode = addQuotesToObjectKeys(objectCode.replace(/\\"|"(?:\\"|[^"])*"|(\/\/.*|\/\*[\s\S]*?\*\/)/g, (m, g) => g ? '' : m));
  // Checking for single quotes and replacing it with " while keeping in mind to not replace single quotes inside double quotes
  objectCode = replaceSingleQuotes(objectCode);
  // console.log(objectCode, ' <==================|++++>> object code');

  return JSON.parse(objectCode);
}
function isEmptyObjectExpression(node) {
  if (types.isObjectExpression(node)) {
    const properties = node.properties;
    if (properties.length === 0) {
      return true;
    } else {
      // Recursively check the properties of the object
      for (const prop of properties) {
        if (!isEmptyObjectExpression(prop.value)) {
          return false;
        }
      }
      return true;
    }
  }
  return false;
}
function removeLiteralPropertiesFromObjectProperties(code) {
  const ast = babel.parse(`var a = ${code}`, {
    presets: [babelPresetTypeScript],
    plugins: ['typescript'],
    sourceType: 'module'
  });
  traverse(ast, {
    ObjectExpression: objectExpressionPath => {
      objectExpressionPath.traverse({
        ObjectProperty(objectPropertyPath) {
          const {
            value
          } = objectPropertyPath.node;
          objectPropertyPath.traverse({
            StringLiteral: stringPath => {
              stringPath;
            }
          });
          if (value.type === 'StringLiteral' || value.type === 'NumericLiteral') {
            objectPropertyPath.remove();
          }
        }
      });
    }
  });
  traverse(ast, {
    ObjectExpression: objectExpressionPath => {
      const properties = objectExpressionPath.node.properties;
      const nonEmptyProperties = properties.filter(prop => !isEmptyObjectExpression(prop.value));
      if (nonEmptyProperties.length !== properties.length) {
        objectExpressionPath.node.properties = nonEmptyProperties;
      }

      // Recursively traverse nested object expressions
      objectExpressionPath.traverse({
        ObjectExpression(innerPath) {
          const innerProperties = innerPath.node.properties;
          const nonEmptyInnerProperties = innerProperties.filter(prop => !isEmptyObjectExpression(prop.value));
          if (nonEmptyInnerProperties.length !== innerProperties.length) {
            innerPath.node.properties = nonEmptyInnerProperties;
          }
        }
      });
    }
  });
  let initAst;
  traverse(ast, {
    VariableDeclarator: variableDeclaratorPath => {
      initAst = variableDeclaratorPath.node.init;
    }
  });
  return initAst;
}
function getIdentifiersObjectFromAstNode(node) {
  const objectCode = generate(node).code;
  return removeLiteralPropertiesFromObjectProperties(objectCode.replace(/\\"|"(?:\\"|[^"])*"|(\/\/.*|\/\*[\s\S]*?\*\/)/g, (m, g) => g ? '' : m));
}
function generateObjectAst(obj) {
  const properties = Object.entries(obj).map(([key, value]) => {
    if (typeof value === 'undefined') {
      return;
    } else if (typeof value === 'object' && !Array.isArray(value)) {
      return types.objectProperty(types.stringLiteral(key), generateObjectAst(value));
    } else if (typeof value === 'object' && Array.isArray(value)) {
      const elements = value.map(val => {
        if (typeof val === 'string') {
          return types.stringLiteral(val);
        } else {
          return generateObjectAst(val);
        }
      });
      return types.objectProperty(types.stringLiteral(key), types.arrayExpression(elements));
    } else if (typeof value === 'boolean') {
      return types.objectProperty(types.stringLiteral(key), types.booleanLiteral(value));
    } else {
      return types.objectProperty(types.stringLiteral(key), typeof value === 'number' ? types.numericLiteral(value) : types.stringLiteral(value));
    }
  });
  return types.objectExpression(properties.filter(property => property));
}
function generateArrayAst(arr) {
  return types.arrayExpression(arr.map(obj => generateObjectAst(obj)));
}
function isImportedFromLibrary(libraries, importName) {
  if (libraries.includes(importName)) {
    return true;
  }
  return false;
}
function isImportFromAbsolutePath(absolutePaths, filePath, importedAbsolutePath) {
  filePath.pop();
  const finalAbsolutePath = path.resolve(filePath.join('/'), importedAbsolutePath);
  if (absolutePaths.includes(finalAbsolutePath)) {
    return true;
  }
  return false;
}
function parseAndExtractConfig(code, opts) {
  try {
    const ast = parseAst(code);
    return extractStyles(ast, opts);
    // return {
    //   code,
    //   styles: {},
    // };
  } catch (err) {
    throw new Error(`Error while parsing and extracting styles. Error: ${err.message} Stack: ${err.stack}.\n*************${opts === null || opts === void 0 ? void 0 : opts.filename}*************\n${code}`);
  }
}
function parseAst(code) {
  try {
    let ast = code;
    if (typeof code === 'string') {
      ast = babel.parse(code, {
        sourceType: 'module',
        presets: [babelPresetTypeScript],
        plugins: [
        // enable jsx and flow syntax
        'typescript', 'jsx']
      });
    }
    return ast;
  } catch (err) {
    throw new Error(`Error while parsing the code. Error: ${err.message} Stack: ${err.stack}`);
  }
}
function extractStyles(ast, opts) {
  function checkWebFileExists(filePath) {
    if (filePath.includes('node_modules')) {
      return false;
    }
    const ext = path.extname(filePath);
    const dirname = path.dirname(filePath);
    const basename = path.basename(filePath, ext);
    const webFilePath = path.join(dirname, `${basename}.web${ext}`);
    return fs.existsSync(webFilePath);
  }
  let styledImportName = '';
  let styledAlias = '';
  let styledAliasImportedName = '';
  let tempPropertyResolverNode;
  let platform = 'all';
  let currentFileName = 'file not found!';
  let outputLibrary;
  let componentSXProp;
  const guessingStyledComponents = [];
  const styled = ['@gluestack-style/react', '@gluestack-ui/themed'];
  const components = ['@gluestack-ui/themed'];
  let configPath;
  let isStyledPathConfigured = false;
  let isComponentsPathConfigured = false;
  const targetPlatform = process.env.GLUESTACK_STYLE_TARGET;
  let createStyleImportedName = '';
  let createComponentsImportedName = '';
  const CREATE_STYLE = 'createStyle';
  const CREATE_COMPONENTS = 'createComponents';
  let cssStyles = {};
  ConfigDefault = opts === null || opts === void 0 ? void 0 : opts.config;
  configThemePath = opts === null || opts === void 0 ? void 0 : opts.configThemePath;
  ConfigDefault = generateMergedThemeTokens(ConfigDefault);
  traverse(ast, {
    ImportDeclaration: importPath => {
      currentFileName = opts === null || opts === void 0 ? void 0 : opts.filename;
      styledAlias = opts === null || opts === void 0 ? void 0 : opts.styledAlias;
      outputLibrary = (opts === null || opts === void 0 ? void 0 : opts.outputLibrary) || outputLibrary;
      if (opts !== null && opts !== void 0 && opts.platform) {
        platform = opts === null || opts === void 0 ? void 0 : opts.platform;
      } else {
        platform = 'all';
      }
      if (!currentFileName.includes('node_modules')) {
        if (currentFileName.includes('.web.')) {
          platform = 'web';
        } else if (checkWebFileExists(currentFileName)) {
          platform = 'native';
        }
      }
      if (opts !== null && opts !== void 0 && opts.styled && Array.isArray(opts === null || opts === void 0 ? void 0 : opts.styled) && !isStyledPathConfigured) {
        styled.push(...(opts === null || opts === void 0 ? void 0 : opts.styled));
        isStyledPathConfigured = true;
      }
      if (opts !== null && opts !== void 0 && opts.components && Array.isArray(opts === null || opts === void 0 ? void 0 : opts.components) && !isComponentsPathConfigured) {
        components.push(...(opts === null || opts === void 0 ? void 0 : opts.components));
        isComponentsPathConfigured = true;
      }
      const importName = importPath.node.source.value;
      const filePath = opts === null || opts === void 0 ? void 0 : opts.filename.split('/');
      if (isImportFromAbsolutePath(components, filePath, importName) || isImportedFromLibrary(components, importName)) {
        importPath.traverse({
          ImportSpecifier(importSpecifierPath) {
            guessingStyledComponents.push(importSpecifierPath.node.local.name);
          }
        });
      }
      if (isImportFromAbsolutePath(styled, filePath, importName) || isImportedFromLibrary(styled, importName)) {
        importPath.traverse({
          ImportSpecifier(importSpecifierPath) {
            if (importSpecifierPath.node.imported.name === 'styled') {
              styledImportName = importSpecifierPath.node.local.name;
            }
            if (importSpecifierPath.node.imported.name === CREATE_STYLE) {
              createStyleImportedName = importSpecifierPath.node.local.name;
            }
            if (importSpecifierPath.node.imported.name === CREATE_COMPONENTS) {
              createComponentsImportedName = importSpecifierPath.node.local.name;
            }
            if (importSpecifierPath.node.imported.name === styledAlias) {
              styledAliasImportedName = importSpecifierPath.node.local.name;
            }
          }
        });
      }
    },
    AssignmentExpression: expressionPath => {
      var _expressionPath$node, _expressionPath$node2;
      if ((expressionPath === null || expressionPath === void 0 || (_expressionPath$node = expressionPath.node) === null || _expressionPath$node === void 0 || (_expressionPath$node = _expressionPath$node.right) === null || _expressionPath$node === void 0 || (_expressionPath$node = _expressionPath$node.callee) === null || _expressionPath$node === void 0 ? void 0 : _expressionPath$node.name) === styledAliasImportedName || (expressionPath === null || expressionPath === void 0 || (_expressionPath$node2 = expressionPath.node) === null || _expressionPath$node2 === void 0 || (_expressionPath$node2 = _expressionPath$node2.right) === null || _expressionPath$node2 === void 0 || (_expressionPath$node2 = _expressionPath$node2.callee) === null || _expressionPath$node2 === void 0 ? void 0 : _expressionPath$node2.name) === styledImportName) {
        var _expressionPath$paren;
        const componentName = expressionPath === null || expressionPath === void 0 || (_expressionPath$paren = expressionPath.parent) === null || _expressionPath$paren === void 0 || (_expressionPath$paren = _expressionPath$paren.id) === null || _expressionPath$paren === void 0 ? void 0 : _expressionPath$paren.name;
        if (componentName) {
          guessingStyledComponents.push(componentName);
        }
      }
    },
    CallExpression: callExpressionPath => {
      var _opts$filename;
      if (!((_opts$filename = opts.filename) !== null && _opts$filename !== void 0 && _opts$filename.includes('node_modules'))) {
        const calleeName = callExpressionPath.node.callee.name;
        if (calleeName === styledAliasImportedName || calleeName === styledImportName || calleeName === createComponentsImportedName || calleeName === createStyleImportedName) {
          callExpressionPath.traverse({
            ObjectProperty(ObjectPath) {
              if (types.isIdentifier(ObjectPath.node.value)) {
                if (ObjectPath.node.value.name === 'undefined') {
                  ObjectPath.remove();
                }
              }
            }
          });
        }
        if (calleeName === styledAliasImportedName || calleeName === styledImportName) {
          var _callExpressionPath$p;
          const componentName = callExpressionPath === null || callExpressionPath === void 0 || (_callExpressionPath$p = callExpressionPath.parent) === null || _callExpressionPath$p === void 0 || (_callExpressionPath$p = _callExpressionPath$p.id) === null || _callExpressionPath$p === void 0 ? void 0 : _callExpressionPath$p.name;
          if (componentName) {
            guessingStyledComponents.push(componentName);
          }
          const args = callExpressionPath.node.arguments;
          const componentThemeNode = args[1];
          // optional case
          const componentConfigNode = args[2] ?? types.objectExpression([]);
          const extendedThemeNode = args[3] ?? types.objectExpression([]);
          if (!(types.isIdentifier(componentThemeNode) || types.isIdentifier(componentConfigNode) || types.isIdentifier(extendedThemeNode))) {
            // args[1] = t.objectExpression([]);

            const extendedThemeNodeProps = [];
            if (extendedThemeNode && extendedThemeNode !== null && extendedThemeNode !== void 0 && extendedThemeNode.properties) {
              extendedThemeNode === null || extendedThemeNode === void 0 || extendedThemeNode.properties.forEach(prop => {
                if (prop.key.name === 'propertyResolver') {
                  tempPropertyResolverNode = prop;
                } else {
                  extendedThemeNodeProps.push(prop);
                }
              });
              extendedThemeNode.properties = extendedThemeNodeProps;
            }
            const theme = getObjectFromAstNode(componentThemeNode);
            const ExtendedConfig = getObjectFromAstNode(extendedThemeNode);
            const componentConfig = getObjectFromAstNode(componentConfigNode);
            if (extendedThemeNode && tempPropertyResolverNode) {
              extendedThemeNode.properties.push(tempPropertyResolverNode);
            }
            const {
              node,
              styles,
              skippedThemeProps
            } = getBuildTimeParams(theme, componentConfig, ExtendedConfig, outputLibrary, platform, 'boot', opts === null || opts === void 0 ? void 0 : opts.disableExtraction);
            cssStyles = {
              ...cssStyles,
              ...styles
            };
            const themeWithIdentifier = getIdentifiersObjectFromAstNode(componentThemeNode);
            const mergedAST = mergeASTs(themeWithIdentifier, skippedThemeProps);
            args[1] = mergedAST;
            if (node) {
              while (args.length < 4) {
                args.push(types.objectExpression([]));
              }
              if (!args[4]) {
                args.push(node);
              } else {
                args[4] = node;
              }
            }
          }

          // console.log(
          //   '<==================|++++>> final ',
          //   generate(path.node).code
          // );
          // console.log(
          //   args,
          //   // resolvedStyles,
          //   // orderedResolved,
          //   // ...path.node.arguments,
          //   // generate(resultParamsNode).code,
          //   // resultParamsNode,
          //   // generate(path.node).code,
          //   'code'
          // );
          // console.log('\n\n >>>>>>>>>>>>>>>>>>>>>\n');

          // console.log('final', generate(path.node).code);
          // console.log('\n >>>>>>>>>>>>>>>>>>>>>\n\n');
        }
        if (calleeName === createStyleImportedName) {
          const args = callExpressionPath.node.arguments;
          const componentThemeNode = args[0];
          const componentConfigNode = args[1] ?? types.objectExpression([]);
          if (!(types.isIdentifier(componentThemeNode) || types.isIdentifier(componentConfigNode))) {
            const theme = getObjectFromAstNode(componentThemeNode);
            const componentConfig = getObjectFromAstNode(componentConfigNode);
            const {
              node,
              styles,
              skippedThemeProps
            } = getBuildTimeParams(theme, componentConfig, {}, outputLibrary, platform, 'extended', opts === null || opts === void 0 ? void 0 : opts.disableExtraction);
            cssStyles = {
              ...cssStyles,
              ...styles
            };
            const themeWithIdentifier = getIdentifiersObjectFromAstNode(componentThemeNode);
            const mergedAST = mergeASTs(themeWithIdentifier, skippedThemeProps);
            args[1] = mergedAST;
            if (node) {
              while (args.length < 3) {
                args.push(types.objectExpression([]));
              }
              if (!args[2]) {
                args.push(node);
              } else {
                args[2] = node;
              }
            }
          }
        }
        if (calleeName === createComponentsImportedName) {
          /*
          extended theme components AST
          {
            box: {
              theme: {},
            },
            button: {
              theme: {},
            },
          }
          */
          const extendedThemeComponents = callExpressionPath.node.arguments[0].properties;
          if (Array.isArray(extendedThemeComponents)) {
            extendedThemeComponents.forEach(property => {
              if (!types.isIdentifier(property.value) && !types.isTemplateLiteral(property.value) && !types.isConditionalExpression(property.value)) {
                const {
                  themeNode,
                  componentConfigNode
                } = findThemeAndComponentConfig(property.value.properties);
                const theme = themeNode ? getObjectFromAstNode(themeNode === null || themeNode === void 0 ? void 0 : themeNode.value) : {};
                const componentConfig = componentConfigNode ? getObjectFromAstNode(componentConfigNode === null || componentConfigNode === void 0 ? void 0 : componentConfigNode.value) : {};
                const {
                  node,
                  styles
                } = getBuildTimeParams(theme, componentConfig, {}, outputLibrary, platform, 'extended', opts === null || opts === void 0 ? void 0 : opts.disableExtraction);
                cssStyles = {
                  ...cssStyles,
                  ...styles
                };
                if (node) {
                  property.value.properties.push(types.objectProperty(types.stringLiteral('BUILD_TIME_PARAMS'), node));
                }
              }
            });
          }
        }
      }
    },
    JSXOpeningElement: jsxOpeningElementPath => {
      if (jsxOpeningElementPath.node.name && jsxOpeningElementPath.node.name.name && guessingStyledComponents.includes(jsxOpeningElementPath.node.name.name)) {
        var _ConfigDefault2, _ConfigDefault3, _componentExtendedCon, _jsxOpeningElementPat;
        const propsToBePersist = [];
        let sxPropsWithIdentifier = {};
        const mergedPropertyConfig = {
          ...((_ConfigDefault2 = ConfigDefault) === null || _ConfigDefault2 === void 0 ? void 0 : _ConfigDefault2.propertyTokenMap),
          ...propertyTokenMap
        };
        const styledSystemProps = {
          ...CSSPropertiesMap,
          ...((_ConfigDefault3 = ConfigDefault) === null || _ConfigDefault3 === void 0 ? void 0 : _ConfigDefault3.aliases)
        };
        const componentExtendedConfig = merge({}, {
          ...ConfigDefault,
          propertyTokenMap: {
            ...mergedPropertyConfig
          }
        });
        const sxPropsConvertedUtilityProps = {};
        const prefixedMediaQueries = {};
        if (componentExtendedConfig !== null && componentExtendedConfig !== void 0 && (_componentExtendedCon = componentExtendedConfig.tokens) !== null && _componentExtendedCon !== void 0 && _componentExtendedCon.mediaQueries) {
          var _componentExtendedCon2;
          Object.keys(componentExtendedConfig === null || componentExtendedConfig === void 0 || (_componentExtendedCon2 = componentExtendedConfig.tokens) === null || _componentExtendedCon2 === void 0 ? void 0 : _componentExtendedCon2.mediaQueries).forEach(key => {
            prefixedMediaQueries[key] = {
              key: `@${key}`,
              isMediaQuery: true
            };
          });
          Object.assign(reservedKeys, {
            ...prefixedMediaQueries
          });
        }
        const attr = jsxOpeningElementPath === null || jsxOpeningElementPath === void 0 || (_jsxOpeningElementPat = jsxOpeningElementPath.node) === null || _jsxOpeningElementPat === void 0 ? void 0 : _jsxOpeningElementPat.attributes;
        attr.forEach((attribute, index) => {
          if (types.isJSXAttribute(attribute)) {
            var _attribute$name;
            const propName = attribute === null || attribute === void 0 || (_attribute$name = attribute.name) === null || _attribute$name === void 0 ? void 0 : _attribute$name.name;
            const propValue = attribute === null || attribute === void 0 ? void 0 : attribute.value;
            if (types.isJSXExpressionContainer(propValue)) {
              if (types.isIdentifier(propValue.expression) || types.isConditionalExpression(propValue.expression)) {
                propsToBePersist.push(attribute);
              } else if (types.isObjectExpression(propValue.expression) && propName !== 'sx') {
                const utilityPropsWithIdentifier = getIdentifiersObjectFromAstNode(propValue.expression);
                const objectProperties = propValue.expression.properties;
                const {
                  result: objectValue
                } = convertExpressionContainerToStaticObject(objectProperties);
                const {
                  prop: propString,
                  propPath,
                  value: utilityPropValue
                } = checkAndReturnUtilityProp(propName, objectValue, styledSystemProps, [], reservedKeys);
                if (propString) {
                  propsToBePersist.push(attribute);
                } else {
                  if (propPath && propPath.length > 0) {
                    setObjectKeyValue(sxPropsConvertedUtilityProps, propPath, utilityPropValue);
                    if (utilityPropsWithIdentifier && utilityPropsWithIdentifier.properties && utilityPropsWithIdentifier.properties.length > 0) {
                      propsToBePersist.push(types.jsxAttribute(types.jsxIdentifier(propName), types.jsxExpressionContainer(utilityPropsWithIdentifier)));
                    }
                  }
                }
              } else {
                if (propName === 'sx') {
                  const objectProperties = propValue.expression.properties;
                  sxPropsWithIdentifier = getIdentifiersObjectFromAstNode(propValue.expression);
                  const {
                    result: sxPropsObject
                  } = convertExpressionContainerToStaticObject(objectProperties);
                  componentSXProp = sxPropsObject;
                } else if (types.isStringLiteral(propValue.expression) || types.isNumericLiteral(propValue.expression)) {
                  const {
                    prop: propString,
                    propPath,
                    value: utilityPropValue
                  } = checkAndReturnUtilityProp(propName, propValue.expression.value, styledSystemProps, [], reservedKeys);
                  if (propString) {
                    propsToBePersist.push(attribute);
                  } else {
                    if (propPath && propPath.length > 0) {
                      setObjectKeyValue(sxPropsConvertedUtilityProps, propPath, utilityPropValue);
                    }
                  }
                } else {
                  propsToBePersist.push(attribute);
                }
              }
            } else {
              const {
                prop: propString,
                propPath,
                value: utilityPropValue
              } = checkAndReturnUtilityProp(propName, propValue === null || propValue === void 0 ? void 0 : propValue.value, styledSystemProps, [], reservedKeys);
              if (propString) {
                propsToBePersist.push(attribute);
              } else {
                if (propPath && propPath.length > 0) {
                  setObjectKeyValue(sxPropsConvertedUtilityProps, propPath, utilityPropValue);
                }
              }
            }
          }
        });
        jsxOpeningElementPath.node.attributes.splice(0, jsxOpeningElementPath.node.attributes.length);
        jsxOpeningElementPath.node.attributes = propsToBePersist;
        const sx = deepMerge({
          ...sxPropsConvertedUtilityProps
        }, {
          ...componentSXProp
        });
        if (Object.keys(sx).length > 0) {
          const verbosedSx = convertSxToSxVerbosed(sx);
          const inlineSxTheme = {
            baseStyle: verbosedSx
          };
          let sxHash = stableHash(sx);
          if (outputLibrary) {
            sxHash = outputLibrary + '-' + sxHash;
          }
          const {
            styledIds,
            verbosedStyleIds
          } = updateOrderUnResolvedMap(inlineSxTheme, sxHash, 'inline', {}, BUILD_TIME_GLUESTACK_STYLESHEET, platform);
          const toBeInjected = BUILD_TIME_GLUESTACK_STYLESHEET.resolve(styledIds, componentExtendedConfig, {}, true, 'inline');
          cssStyles = {
            ...cssStyles,
            ...toBeInjected
          };
          const current_global_map = BUILD_TIME_GLUESTACK_STYLESHEET.getStyleMap();
          const orderedResolvedTheme = [];
          current_global_map === null || current_global_map === void 0 || current_global_map.forEach(styledResolved => {
            var _styledResolved$meta2;
            if (styledIds.includes(styledResolved === null || styledResolved === void 0 || (_styledResolved$meta2 = styledResolved.meta) === null || _styledResolved$meta2 === void 0 ? void 0 : _styledResolved$meta2.cssId)) {
              orderedResolvedTheme.push(styledResolved);
            }
          });
          const styleIdsAst = generateObjectAst(verbosedStyleIds);
          const orderResolvedArrayExpression = [];
          orderedResolvedTheme.forEach(styledResolved => {
            delete styledResolved.toBeInjected;
            if (targetPlatform !== 'web') {
              delete styledResolved.original;
              delete styledResolved.meta.cssRulesSet;
              delete styledResolved.meta.weight;
              delete styledResolved.type;
              delete styledResolved.componentHash;
              delete styledResolved.extendedConfig;
              delete styledResolved.value;
            }
            const orderedResolvedAst = generateObjectAst(styledResolved);
            orderResolvedArrayExpression.push(orderedResolvedAst);
          });
          jsxOpeningElementPath.node.attributes.push(types.jsxAttribute(types.jsxIdentifier('verbosedStyleIds'), types.jsxExpressionContainer(styleIdsAst)));
          jsxOpeningElementPath.node.attributes.push(types.jsxAttribute(types.jsxIdentifier('orderedResolved'), types.jsxExpressionContainer(types.arrayExpression(orderResolvedArrayExpression))));
          if (opts !== null && opts !== void 0 && opts.disableExtraction) {
            jsxOpeningElementPath.node.attributes.push(types.jsxAttribute(types.jsxIdentifier('disableExtraction'), types.jsxExpressionContainer(types.booleanLiteral(true))));
          }
        }
        if (sxPropsWithIdentifier && sxPropsWithIdentifier.properties && sxPropsWithIdentifier.properties.length > 0) {
          jsxOpeningElementPath.node.attributes.push(types.jsxAttribute(types.jsxIdentifier('sx'), types.jsxExpressionContainer(sxPropsWithIdentifier)));
        }
        componentSXProp = undefined;
      }
    }
  });
  return {
    ast,
    code: generate(ast).code,
    styles: cssStyles
  };
}
//# sourceMappingURL=extract-styles.js.map